==> apache-nuttx-10.0.1-incubating.tar.gz: nuttx/boards/arm/stm32/hymini-stm32v/src/stm32_leds.c <==
/****************************************************************************
 * boards/arm/stm32/hymini-stm32v/src/stm32_leds.c
 *
 *   Copyright (C) 2009, 2011, 2013, 2015 Gregory Nutt. All rights reserved.
 *   Author: Gregory Nutt <gnutt@nuttx.org>
 *           Laurent Latil <laurent@latil.nom.fr>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 3. Neither the name NuttX nor the names of its contributors may be
 *    used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 ****************************************************************************/

/****************************************************************************
 * Included Files
 ****************************************************************************/

#include <nuttx/config.h>

#include <stdint.h>
#include <stdbool.h>
#include <debug.h>

#include <nuttx/board.h>

#include "chip.h"
#include "arm_arch.h"
#include "arm_internal.h"
#include "stm32.h"
#include "hymini-stm32v.h"

#include <arch/board/board.h>  /* Should always be included last due to dependencies */

/****************************************************************************
 * Pre-processor Definitions
 ****************************************************************************/

/* The following definitions map the encoded LED setting to GPIO settings */

#define HYMINI_STM32_LED1     (1 << 0)
#define HYMINI_STM32_LED2     (1 << 1)

#define ON_SETBITS_SHIFT  (0)
#define ON_CLRBITS_SHIFT  (4)
#define OFF_SETBITS_SHIFT (8)
#define OFF_CLRBITS_SHIFT (12)

#define ON_BITS(v)        ((v) & 0xff)
#define OFF_BITS(v)       (((v) >> 8) & 0x0ff)
#define SETBITS(b)        ((b) & 0x0f)
#define CLRBITS(b)        (((b) >> 4) & 0x0f)

#define ON_SETBITS(v)     (SETBITS(ON_BITS(v))
#define ON_CLRBITS(v)     (CLRBITS(ON_BITS(v))
#define OFF_SETBITS(v)    (SETBITS(OFF_BITS(v))
#define OFF_CLRBITS(v)    (CLRBITS(OFF_BITS(v))

/* On: !LED1 + !LED2   Off: - */

#define LED_STARTED_ON_SETBITS       ((0) << ON_SETBITS_SHIFT)
#define LED_STARTED_ON_CLRBITS       ((HYMINI_STM32_LED1|HYMINI_STM32_LED2) << ON_CLRBITS_SHIFT)
#define LED_STARTED_OFF_SETBITS      (0 << OFF_SETBITS_SHIFT)
#define LED_STARTED_OFF_CLRBITS      (0 << OFF_CLRBITS_SHIFT)

/* On: LED1+!LED2   Off: N/A */

#define LED_HEAPALLOCATE_ON_SETBITS  ((HYMINI_STM32_LED1) << ON_SETBITS_SHIFT)
#define LED_HEAPALLOCATE_ON_CLRBITS  ((HYMINI_STM32_LED2) << ON_CLRBITS_SHIFT)
#define LED_HEAPALLOCATE_OFF_SETBITS (0)
#define LED_HEAPALLOCATE_OFF_CLRBITS (0)

/* On: LED2+!LED1   Off: N/A */

#define LED_IRQSENABLED_ON_SETBITS   ((HYMINI_STM32_LED2) << ON_SETBITS_SHIFT)
#define LED_IRQSENABLED_ON_CLRBITS   ((HYMINI_STM32_LED1) << ON_CLRBITS_SHIFT)
#define LED_IRQSENABLED_OFF_SETBITS  (0)
#define LED_IRQSENABLED_OFF_CLRBITS  (0)

/* On: LED1+!LED2   Off: N/A */

#define LED_STACKCREATED_ON_SETBITS  ((HYMINI_STM32_LED1) << ON_SETBITS_SHIFT)
#define LED_STACKCREATED_ON_CLRBITS  ((HYMINI_STM32_LED2) << ON_CLRBITS_SHIFT)
#define LED_STACKCREATED_OFF_SETBITS (0)
#define LED_STACKCREATED_OFF_CLRBITS (0)

/* On: !LED1   Off: LED1 */

#define LED_INIRQ_ON_SETBITS         ((0) << ON_SETBITS_SHIFT)
#define LED_INIRQ_ON_CLRBITS         ((HYMINI_STM32_LED1) << ON_CLRBITS_SHIFT)
#define LED_INIRQ_OFF_SETBITS        ((HYMINI_STM32_LED1) << OFF_SETBITS_SHIFT)
#define LED_INIRQ_OFF_CLRBITS        ((0) << OFF_CLRBITS_SHIFT)

/* On: LED2   Off: !LED2 */

#define LED_SIGNAL_ON_SETBITS        ((HYMINI_STM32_LED2) << ON_SETBITS_SHIFT)
#define LED_SIGNAL_ON_CLRBITS        ((0) << ON_CLRBITS_SHIFT)
#define LED_SIGNAL_OFF_SETBITS       ((0) << OFF_SETBITS_SHIFT)
#define LED_SIGNAL_OFF_CLRBITS       ((HYMINI_STM32_LED2) << OFF_CLRBITS_SHIFT)

/* On: LED1+LED2   Off: - */

#define LED_ASSERTION_ON_SETBITS     ((HYMINI_STM32_LED2|HYMINI_STM32_LED2) << ON_SETBITS_SHIFT)
#define LED_ASSERTION_ON_CLRBITS     ((0) << ON_CLRBITS_SHIFT)
#define LED_ASSERTION_OFF_SETBITS    ((0) << OFF_SETBITS_SHIFT)
#define LED_ASSERTION_OFF_CLRBITS    ((0) << OFF_CLRBITS_SHIFT)

/* On: LED1   Off: LED2 */

#define LED_PANIC_ON_SETBITS         ((HYMINI_STM32_LED1) << ON_SETBITS_SHIFT)
#define LED_PANIC_ON_CLRBITS         ((HYMINI_STM32_LED2) << ON_CLRBITS_SHIFT)
#define LED_PANIC_OFF_SETBITS        ((HYMINI_STM32_LED2) << OFF_SETBITS_SHIFT)
#define LED_PANIC_OFF_CLRBITS        ((HYMINI_STM32_LED1) << OFF_CLRBITS_SHIFT)

/****************************************************************************
 * Private Data
 ****************************************************************************/

static const uint16_t g_ledbits[8] =
{
  (LED_STARTED_ON_SETBITS | LED_STARTED_ON_CLRBITS | LED_STARTED_OFF_SETBITS
   | LED_STARTED_OFF_CLRBITS),

  (LED_HEAPALLOCATE_ON_SETBITS | LED_HEAPALLOCATE_ON_CLRBITS
   | LED_HEAPALLOCATE_OFF_SETBITS | LED_HEAPALLOCATE_OFF_CLRBITS),

  (LED_IRQSENABLED_ON_SETBITS | LED_IRQSENABLED_ON_CLRBITS
   | LED_IRQSENABLED_OFF_SETBITS | LED_IRQSENABLED_OFF_CLRBITS),

  (LED_STACKCREATED_ON_SETBITS | LED_STACKCREATED_ON_CLRBITS
   | LED_STACKCREATED_OFF_SETBITS | LED_STACKCREATED_OFF_CLRBITS),

  (LED_INIRQ_ON_SETBITS | LED_INIRQ_ON_CLRBITS | LED_INIRQ_OFF_SETBITS
   | LED_INIRQ_OFF_CLRBITS),

  (LED_SIGNAL_ON_SETBITS | LED_SIGNAL_ON_CLRBITS | LED_SIGNAL_OFF_SETBITS
   | LED_SIGNAL_OFF_CLRBITS),

  (LED_ASSERTION_ON_SETBITS | LED_ASSERTION_ON_CLRBITS | LED_ASSERTION_OFF_SETBITS
   | LED_ASSERTION_OFF_CLRBITS),

  (LED_PANIC_ON_SETBITS | LED_PANIC_ON_CLRBITS | LED_PANIC_OFF_SETBITS
   | LED_PANIC_OFF_CLRBITS)
};

/****************************************************************************
 * Private Functions
 ****************************************************************************/

static inline void led_clrbits(unsigned int clrbits)
{
  if ((clrbits & HYMINI_STM32_LED1) != 0)
  {
    stm32_gpiowrite(GPIO_LED1, false);
  }

  if ((clrbits & HYMINI_STM32_LED2) != 0)
  {
    stm32_gpiowrite(GPIO_LED2, false);
  }
}

static inline void led_setbits(unsigned int setbits)
{
  if ((setbits & HYMINI_STM32_LED1) != 0)
  {
    stm32_gpiowrite(GPIO_LED1, true);
  }

  if ((setbits & HYMINI_STM32_LED2) != 0)
  {
    stm32_gpiowrite(GPIO_LED2, true);
  }
}

static void led_setonoff(unsigned int bits)
{
  led_clrbits(CLRBITS(bits));
  led_setbits(SETBITS(bits));
}

/****************************************************************************
 * Public Functions
 ****************************************************************************/

/****************************************************************************
 * Name: board_autoled_initialize
 ****************************************************************************/

#ifdef CONFIG_ARCH_LEDS
void board_autoled_initialize(void)
{
  /* Configure LED1 & LED2 GPIOs for output */
  stm32_configgpio(GPIO_LED1);
  stm32_configgpio(GPIO_LED2);
}

/****************************************************************************
 * Name: board_autoled_on
 ****************************************************************************/

void board_autoled_on(int led)
{
  led_setonoff(ON_BITS(g_ledbits[led]));
}

/****************************************************************************
 * Name: board_autoled_off
 ****************************************************************************/

void board_autoled_off(int led)
{
  led_setonoff(OFF_BITS(g_ledbits[led]));
}

#endif /* CONFIG_ARCH_LEDS */

==> apache-nuttx-10.2.0-incubating.tar.gz: nuttx/boards/arm/stm32/hymini-stm32v/src/stm32_leds.c <==
/****************************************************************************
 * boards/arm/stm32/hymini-stm32v/src/stm32_leds.c
 *
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.  The
 * ASF licenses this file to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance with the
 * License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the
 * License for the specific language governing permissions and limitations
 * under the License.
 *
 ****************************************************************************/

/****************************************************************************
 * Included Files
 ****************************************************************************/

#include <nuttx/config.h>

#include <stdint.h>
#include <stdbool.h>
#include <debug.h>

#include <nuttx/board.h>

#include "chip.h"
#include "arm_arch.h"
#include "arm_internal.h"
#include "stm32.h"
#include "hymini-stm32v.h"

#include <arch/board/board.h>  /* Should always be included last due to dependencies */

/****************************************************************************
 * Pre-processor Definitions
 ****************************************************************************/

/* The following definitions map the encoded LED setting to GPIO settings */

#define HYMINI_STM32_LED1     (1 << 0)
#define HYMINI_STM32_LED2     (1 << 1)

#define ON_SETBITS_SHIFT  (0)
#define ON_CLRBITS_SHIFT  (4)
#define OFF_SETBITS_SHIFT (8)
#define OFF_CLRBITS_SHIFT (12)

#define ON_BITS(v)        ((v) & 0xff)
#define OFF_BITS(v)       (((v) >> 8) & 0x0ff)
#define SETBITS(b)        ((b) & 0x0f)
#define CLRBITS(b)        (((b) >> 4) & 0x0f)

#define ON_SETBITS(v)     (SETBITS(ON_BITS(v))
#define ON_CLRBITS(v)     (CLRBITS(ON_BITS(v))
#define OFF_SETBITS(v)    (SETBITS(OFF_BITS(v))
#define OFF_CLRBITS(v)    (CLRBITS(OFF_BITS(v))

/* On: !LED1 + !LED2   Off: - */

#define LED_STARTED_ON_SETBITS       ((0) << ON_SETBITS_SHIFT)
#define LED_STARTED_ON_CLRBITS       ((HYMINI_STM32_LED1|HYMINI_STM32_LED2) << ON_CLRBITS_SHIFT)
#define LED_STARTED_OFF_SETBITS      (0 << OFF_SETBITS_SHIFT)
#define LED_STARTED_OFF_CLRBITS      (0 << OFF_CLRBITS_SHIFT)

/* On: LED1+!LED2   Off: N/A */

#define LED_HEAPALLOCATE_ON_SETBITS  ((HYMINI_STM32_LED1) << ON_SETBITS_SHIFT)
#define LED_HEAPALLOCATE_ON_CLRBITS  ((HYMINI_STM32_LED2) << ON_CLRBITS_SHIFT)
#define LED_HEAPALLOCATE_OFF_SETBITS (0)
#define LED_HEAPALLOCATE_OFF_CLRBITS (0)

/* On: LED2+!LED1   Off: N/A */

#define LED_IRQSENABLED_ON_SETBITS   ((HYMINI_STM32_LED2) << ON_SETBITS_SHIFT)
#define LED_IRQSENABLED_ON_CLRBITS   ((HYMINI_STM32_LED1) << ON_CLRBITS_SHIFT)
#define LED_IRQSENABLED_OFF_SETBITS  (0)
#define LED_IRQSENABLED_OFF_CLRBITS  (0)

/* On: LED1+!LED2   Off: N/A */

#define LED_STACKCREATED_ON_SETBITS  ((HYMINI_STM32_LED1) << ON_SETBITS_SHIFT)
#define LED_STACKCREATED_ON_CLRBITS  ((HYMINI_STM32_LED2) << ON_CLRBITS_SHIFT)
#define LED_STACKCREATED_OFF_SETBITS (0)
#define LED_STACKCREATED_OFF_CLRBITS (0)

/* On: !LED1   Off: LED1 */

#define LED_INIRQ_ON_SETBITS         ((0) << ON_SETBITS_SHIFT)
#define LED_INIRQ_ON_CLRBITS         ((HYMINI_STM32_LED1) << ON_CLRBITS_SHIFT)
#define LED_INIRQ_OFF_SETBITS        ((HYMINI_STM32_LED1) << OFF_SETBITS_SHIFT)
#define LED_INIRQ_OFF_CLRBITS        ((0) << OFF_CLRBITS_SHIFT)

/* On: LED2   Off: !LED2 */

#define LED_SIGNAL_ON_SETBITS        ((HYMINI_STM32_LED2) << ON_SETBITS_SHIFT)
#define LED_SIGNAL_ON_CLRBITS        ((0) << ON_CLRBITS_SHIFT)
#define LED_SIGNAL_OFF_SETBITS       ((0) << OFF_SETBITS_SHIFT)
#define LED_SIGNAL_OFF_CLRBITS       ((HYMINI_STM32_LED2) << OFF_CLRBITS_SHIFT)

/* On: LED1+LED2   Off: - */

#define LED_ASSERTION_ON_SETBITS     ((HYMINI_STM32_LED2|HYMINI_STM32_LED2) << ON_SETBITS_SHIFT)
#define LED_ASSERTION_ON_CLRBITS     ((0) << ON_CLRBITS_SHIFT)
#define LED_ASSERTION_OFF_SETBITS    ((0) << OFF_SETBITS_SHIFT)
#define LED_ASSERTION_OFF_CLRBITS    ((0) << OFF_CLRBITS_SHIFT)

/* On: LED1   Off: LED2 */

#define LED_PANIC_ON_SETBITS         ((HYMINI_STM32_LED1) << ON_SETBITS_SHIFT)
#define LED_PANIC_ON_CLRBITS         ((HYMINI_STM32_LED2) << ON_CLRBITS_SHIFT)
#define LED_PANIC_OFF_SETBITS        ((HYMINI_STM32_LED2) << OFF_SETBITS_SHIFT)
#define LED_PANIC_OFF_CLRBITS        ((HYMINI_STM32_LED1) << OFF_CLRBITS_SHIFT)

/****************************************************************************
 * Private Data
 ****************************************************************************/

static const uint16_t g_ledbits[8] =
{
  (LED_STARTED_ON_SETBITS | LED_STARTED_ON_CLRBITS |
  LED_STARTED_OFF_SETBITS | LED_STARTED_OFF_CLRBITS),

  (LED_HEAPALLOCATE_ON_SETBITS | LED_HEAPALLOCATE_ON_CLRBITS |
  LED_HEAPALLOCATE_OFF_SETBITS | LED_HEAPALLOCATE_OFF_CLRBITS),

  (LED_IRQSENABLED_ON_SETBITS | LED_IRQSENABLED_ON_CLRBITS |
  LED_IRQSENABLED_OFF_SETBITS | LED_IRQSENABLED_OFF_CLRBITS),

  (LED_STACKCREATED_ON_SETBITS | LED_STACKCREATED_ON_CLRBITS |
  LED_STACKCREATED_OFF_SETBITS | LED_STACKCREATED_OFF_CLRBITS),

  (LED_INIRQ_ON_SETBITS | LED_INIRQ_ON_CLRBITS |
  LED_INIRQ_OFF_SETBITS | LED_INIRQ_OFF_CLRBITS),

  (LED_SIGNAL_ON_SETBITS | LED_SIGNAL_ON_CLRBITS |
  LED_SIGNAL_OFF_SETBITS | LED_SIGNAL_OFF_CLRBITS),

  (LED_ASSERTION_ON_SETBITS | LED_ASSERTION_ON_CLRBITS |
  LED_ASSERTION_OFF_SETBITS | LED_ASSERTION_OFF_CLRBITS),

  (LED_PANIC_ON_SETBITS | LED_PANIC_ON_CLRBITS |
  LED_PANIC_OFF_SETBITS | LED_PANIC_OFF_CLRBITS)
};

/****************************************************************************
 * Private Functions
 ****************************************************************************/

static inline void led_clrbits(unsigned int clrbits)
{
  if ((clrbits & HYMINI_STM32_LED1) != 0)
    {
      stm32_gpiowrite(GPIO_LED1, false);
    }

  if ((clrbits & HYMINI_STM32_LED2) != 0)
    {
      stm32_gpiowrite(GPIO_LED2, false);
    }
}

static inline void led_setbits(unsigned int setbits)
{
  if ((setbits & HYMINI_STM32_LED1) != 0)
    {
      stm32_gpiowrite(GPIO_LED1, true);
    }

  if ((setbits & HYMINI_STM32_LED2) != 0)
    {
      stm32_gpiowrite(GPIO_LED2, true);
    }
}

static void led_setonoff(unsigned int bits)
{
  led_clrbits(CLRBITS(bits));
  led_setbits(SETBITS(bits));
}

/****************************************************************************
 * Public Functions
 ****************************************************************************/

/****************************************************************************
 * Name: board_autoled_initialize
 ****************************************************************************/

#ifdef CONFIG_ARCH_LEDS
void board_autoled_initialize(void)
{
  /* Configure LED1 & LED2 GPIOs for output */

  stm32_configgpio(GPIO_LED1);
  stm32_configgpio(GPIO_LED2);
}

/****************************************************************************
 * Name: board_autoled_on
 ****************************************************************************/

void board_autoled_on(int led)
{
  led_setonoff(ON_BITS(g_ledbits[led]));
}

/****************************************************************************
 * Name: board_autoled_off
 ****************************************************************************/

void board_autoled_off(int led)
{
  led_setonoff(OFF_BITS(g_ledbits[led]));
}

#endif /* CONFIG_ARCH_LEDS */
